//
//  GSPathSegment.h
//  PathOperations
//
//  Created by Georg Seifert on 03.03.14.
//  Copyright (c) 2014 Georg Seifert. All rights reserved.
//

@class FTPointArray;

#import <GlyphsCore/GSNode.h>

typedef struct {
	NSPoint elements[4];
	uint8 count;
	GSNodeType type;
	BOOL black; // 0 == white(counter) 1 == black
	CGFloat minX;
	CGFloat maxX;
	CGFloat minY;
	CGFloat maxY;
} GSSegmentStruct;

void _GSSegmentUpdateBounds(GSSegmentStruct*S);

@interface GSPathSegment : NSObject <NSCopying> {
	
@public
	NSPoint elements[40];
	uint8 type;
	uint8 count;
	BOOL black; // 0 == white(counter) 1 == black
	CGFloat minX;
	CGFloat maxX;
	CGFloat minY;
	CGFloat maxY;
}
@property (nonatomic) uint8 type;
@property (nonatomic) uint8 countOfPoints;
@property (strong, nonatomic) NSArray* objects;

- (instancetype) initWithLinePoint1:(NSPoint)P1 point2:(NSPoint)P2 direction:(BOOL) Black;
- (instancetype) initWithCurvePoint1:(NSPoint)P1 point2:(NSPoint)P2 point3:(NSPoint)P3 point4:(NSPoint)P4 direction:(BOOL) Black;
- (instancetype) initWithDescription:(NSString*)string;
- (instancetype) initWithQCurve:(NSArray *)Points;
- (instancetype) initWithQCurve:(NSPoint *)Points count:(int)Count;
- (instancetype) initWithSegmentStruct:(GSSegmentStruct*)S ;
- (GSSegmentStruct)segmentStruct ;
- (void) updateBounds;
- (void) reverse;
- (NSComparisonResult) compare:(GSPathSegment*) otherObject;
- (BOOL) isEqualToSegment:(GSPathSegment*) otherSegment;
- (NSPoint) middlePoint;
- (NSPoint) middlePointRight;
- (NSPoint) middlePointLeft;
- (void) splitFirstHalf:(GSPathSegment**)left secondHalf:(GSPathSegment**)right;
- (void)splitAtTime:(CGFloat)t firstHalf:(GSPathSegment **)left secondHalf:(GSPathSegment **)right;
- (bool) intersects:(GSPathSegment*) OtherSegment;
- (FTPointArray*)intersectionPoints:(GSPathSegment*)otherSegment;
- (FTPointArray*)intersectionPoints:(GSPathSegment*)otherSegment unlimited:(BOOL)unlimited;
- (NSArray*)inflectionPoints;
- (NSPoint)pointAtIndex:(NSUInteger)idx;
- (void)setPoint:(NSPoint)p atIndex:(NSUInteger)idx;
- (CGFloat)curvatureAtTime:(CGFloat)t;
- (NSPoint)pointAtTime:(CGFloat)t;
- (NSPoint)normalAtTime:(CGFloat)t;
- (CGFloat)length;
//- (double**) intersections:(GSPathSegment*) OtherSegment NS_RETURNS_INNER_POINTER;
@end
